/*
 * Created by Brett on 17/11/23.
 * Licensed under GNU General Public License V3.0
 * See LICENSE file for license detail
 */
#include <bf_mips_codegen.h>
#include <iostream>

std::string preamble = R"(.data
	data_pointer:   .word   0
.text
setup:
	li $v0, 9
	li $a0, 30000
	syscall
	
	la $t1, data_pointer
	sw $v0, ($t1)
	
	# t0 - current address (data_pointer)
	move $t0, $v0
bf:
)";

void process_print(const std::vector<token_t>& tokens, size_t index);

void codegen(bf_tokenizer& tokenizer, std::ostream& out)
{
    out << preamble;
    if (!tokenizer.hasNext())
        throw std::runtime_error("You failed to provide valid BF code");
    // skip past comments
    if (tokenizer.peek().type == bf_token::OPEN)
        while (tokenizer.hasNext() && tokenizer.peek().type != bf_token::CLOSE)
            tokenizer.advance();
    
    tokenizer.print();
    
    while (tokenizer.hasNext())
    {
        auto& token = tokenizer.next();
        uint64_t name = 0;
        if (token.name.has_value())
            name = token.name.value();
        switch (token.type)
        {
            case bf_token::INC_DP:
                out       << "\taddi  $t0, $t0, " << token.offset << "\n";
                break;
            case bf_token::DEC_DP:
                out       << "\tsubi  $t0, $t0, " << token.offset << "\n";
                break;
            case bf_token::INC_DV:
                out       << "\tlb    $t1, ($t0)\n"
                          << "\taddi  $t1, $t1, " << static_cast<uint8_t>(token.offset) << "\n"
                          << "\tsb    $t1, ($t0)\n";
                break;
            case bf_token::DEC_DV:
                out << "\tlb    $t1, ($t0)\n"
                          << "\tsubi  $t1, $t1, " << static_cast<uint8_t>(token.offset) << "\n"
                          << "\tsb    $t1, ($t0)\n";
                break;
            case bf_token::PRINT:
                out       << "\tli    $v0, 11\n"
                          << "\tlb    $a0, ($t0)\n"
                          << "\tsyscall\n";
                break;
            case bf_token::READ:
                out       << "\tli    $v0, 12\n"
                          << "\tsyscall\n"
                          << "\tsb    $v0, ($t0)\n";
                 break;
            case bf_token::OPEN:
                out       << "\tlb    $t1, ($t0)\n"
                          << "\tbeqz  $t1, BF_CLOSE_" << name << '\n'
                          << "BF_OPEN_" << name << ":\n";
                break;
            case bf_token::CLOSE:
                out       << "\tlb    $t1, ($t0)\n"
                          << "\tbnez  $t1, BF_OPEN_" << name << '\n'
                          << "BF_CLOSE_" << name << ":\n";
                break;
        }
    }
}

